@@ -23,6 +23,10 @@ class Irradiance():
     import pandas as pd
     import numpy as np
     from pvmodule import PVGIS
+    from scipy.signal import savgol_filter
+    import warnings
+
+    warnings.simplefilter(action='ignore', category=pd.errors.PerformanceWarning)
 
     #params
 
@@ -51,7 +55,7 @@ class Irradiance():
                            'T2m':'2m Air Temperature',
                            'WS10m':'10m Wind speed'
                           }, inplace = True)
-    data = data.drop(['Int'], axis=1)
+    data = data.drop(['Int','H_sun'], axis=1)
 
     
     
@@ -73,17 +77,33 @@ class Irradiance():
     data['Month'] = data.index.strftime("%m").astype(float)
     data['Day'] = data.index.strftime("%d").astype(float)
 
+
+
     #new_index = data.index.map(lambda t: t.replace(year=2030))
     #data=data.set_index(new_index)
     #display(data)
+    if startyear != endyear:
+      data.index = pd.DatetimeIndex(data.index)
+      data.index = data.index + pd.DateOffset(year=2030)
+      data = data.groupby(data.index, as_index=True).mean()
+      tp_year = 2030
+    else:
+      tp_year = startyear
+    #data['Group_By'] = pd.to_datetime(data['Group_By'], format='2030-%m-%d %H:%M:%S')
 
 
 
 
-    df_both = pd.date_range("2020-01-01 00:00:00", "2020-12-31 23:00:00", freq='5T').to_frame()
+    #####
+
+
+    df_both = pd.date_range(f"{tp_year}-01-01 00:10:00", f"{tp_year}-12-31 23:10:00", freq='5T').to_frame()
     df_both = df_both.drop([0], axis=1)
     df_both = df_both.merge(data, left_index=True, right_index=True, how='left')
 
+
+
+
         
     df_both['Time_H'] = df_both.index.strftime("%H").astype(float)
     df_both['Time_M'] = df_both.index.strftime("%M").astype(float)
@@ -92,7 +112,11 @@ class Irradiance():
     df_both['Month']= df_both['Month'].fillna(method='ffill')
     df_both['Day']= df_both['Day'].fillna(method='ffill')
 
-    data = df_both.interpolate(method='polynomial', order=5)
+    data = df_both.interpolate(method='polynomial', order=2)
+    data['DHI'] = data['DHI'].round(0)
+    data['GHI'] = data['GHI'].round(0)
+    data['DNI'] = data['DNI'].round(0)
+
 
 
     DOY =  pd.DatetimeIndex(data.index.values).day_of_year
@@ -163,6 +187,9 @@ class Irradiance():
     if module['BIPV'] == 'N' :
       data['Total_G'] = G_front
       data[['GHI', 'DHI','DNI','G_Front','Total_G']] = data[['GHI', 'DHI','DNI','G_Front','Total_G']].clip(lower=0)
+      data['Total_G'] = data['Total_G'].round(0)
+      #data = data.drop(['Declination', 'Hour angle','Rb_front','Rb_rear','Solar Zenith angle','GHI','DNI','DHI','2m Air Temperature','10m Wind speed'], axis=1)
+
 
       return inputs ,data, metadata
 
@@ -171,16 +198,15 @@ class Irradiance():
     GR_diffuse =  Irradiance()._GR_diffuse(data, module, panel_distance, panel_tilt)
     GR_reflected = Irradiance()._GR_reflected(data, albedo, module, panel_distance, panel_tilt, azimuth,Elevation)
 
-    data['GR_beam'] = GR_beam
-    data['GR_diffuse'] = GR_diffuse
-    data['GR_reflected'] = GR_reflected
-
-
-
     G_Rear = GR_beam + GR_diffuse + GR_reflected
+
+    G_Rear = Irradiance()._PV_SOL_Rear(data, albedo, module, panel_distance, panel_tilt, azimuth,Elevation)
     data['G_Rear'] = G_Rear
     data['Total_G'] = G_Rear + G_front
     data[['GHI', 'DHI','DNI','G_Front','Total_G','G_Rear']] = data[['GHI', 'DHI','DNI','G_Front','Total_G','G_Rear']].clip(lower=0)
+    data['Total_G'] = data['Total_G'].round(0)
+    #data = data.drop(['Declination', 'Hour angle','Rb_front','Rb_rear','Solar Zenith angle','GHI','DNI','DHI','2m Air Temperature','10m Wind speed'], axis=1)
+
 
     return inputs ,data, metadata
 
@@ -436,13 +462,41 @@ class Irradiance():
 
     return GR_reflected
 
+  def _PV_SOL_Rear(self, data, albedo, module, panel_distance, panel_tilt, azimuth, Elevation):
+    #https://help.valentin-software.com/pvsol/en/calculation/pv-modules/bifacial-modules/
+    import numpy as np
 
+    Xm = 0
+    h = Elevation
+    d = panel_distance
+    b = module['height']
 
+    x_start = Xm + - (h/ np.tan(panel_tilt*np.pi/180))
+    x_end = Xm + d + (h*(d - b * np.cos(panel_tilt*np.pi/180)))/(b*np.sin(panel_tilt*np.pi/180))
 
+    x = x_end - x_start
 
-    #VF_Module2Ground = Irradiance()._VF_front_Module2sGround(data, module, panel_distance, panel_tilt,azimuth,Elevation) + Irradiance()._VF_front_Module2usGround(data, module, panel_distance, panel_tilt,azimuth,Elevation)
+    tan_delta_1v = (h + b*np.sin(panel_tilt*np.pi/180))/(b * np.cos(panel_tilt*np.pi/180) - d - x)
+    tan_delta_1h = h / x
 
-    #GF_reflected = data['GHI']*albedo*VF_Module2Ground['Shadow']
+    tan_delta_2v = (h + b*np.sin(panel_tilt*np.pi/180))/(x + b*np.cos(panel_tilt*np.pi/180))
+    tan_delta_2h =  h / (x + d)
+
+    E_diffuse_test_point = data['DHI']*((tan_delta_1h - tan_delta_1v) + (tan_delta_2h + tan_delta_2v)) / (np.pi/2)
+
+    E_in_test_point = data['DHI'] + data['GHI']
+
+    E_out_test_point = albedo * E_in_test_point
+
+    E_rear_ground = E_out_test_point * (tan_delta_1h + tan_delta_2v)/(np.pi/180) * ((tan_delta_1h + tan_delta_2v)/(2)-panel_tilt*np.pi/180)
+
+    tany_next = (b*np.sin(panel_tilt*np.pi/180))/(d + b*np.cos(panel_tilt*np.pi/180))
+
+    E_rear_diffuse = data['DHI'] * (panel_tilt*np.pi/180 - tany_next)(np.pi/180)
+
+    E_rear = E_rear_diffuse + E_rear_ground
+
+    return E_rear
 
 
 
